@@ -10,6 +10,8 @@
 #define BOLT_PASSES_SHRINKWRAPPING_H
 
 #include "bolt/Passes/FrameAnalysis.h"
+#include "llvm/MC/MCRegisterInfo.h"
+#include <atomic>
 
 namespace llvm {
 namespace bolt {
@@ -302,13 +304,16 @@ class ShrinkWrapping {
   std::vector<int64_t> PopOffsetByReg;
   std::vector<MCPhysReg> DomOrder;
   CalleeSavedAnalysis CSA;
-  std::vector<SmallSetVector<MCInst *, 4>> SavePos;
-  std::vector<uint64_t> BestSaveCount;
-  std::vector<MCInst *> BestSavePos;
+  std::vector<std::vector<uint64_t>> BestSaveCount;
+  std::vector<std::vector<MCInst *>> BestSavePos;
 
   /// Pass stats
-  static uint64_t SpillsMovedRegularMode;
-  static uint64_t SpillsMovedPushPopMode;
+  static std::atomic<std::uint64_t> SpillsMovedRegularMode;
+  static std::atomic<std::uint64_t> SpillsMovedPushPopMode;
+  static std::atomic<std::uint64_t> SpillsMovedDynamicCount;
+  static std::atomic<std::uint64_t> SpillsFailedDynamicCount;
+  static std::atomic<std::uint64_t> InstrDynamicCount;
+  static std::atomic<std::uint64_t> StoreDynamicCount;
 
   Optional<unsigned> AnnotationIndex;
 
@@ -514,7 +519,7 @@ public:
         BC.MIB->removeAnnotation(Inst, getAnnotationIndex());
   }
 
-  bool perform();
+  bool perform(bool HotOnly = false);
 
   static void printStats();
 };
