@@ -1,6 +1,6 @@
 // Test returning a single element aggregate value containing a double.
-// RUN: %clang_cc1 -triple i686-linux %s -emit-llvm -o - | FileCheck %s --check-prefix=X86_32
-// RUN: %clang_cc1 %s -emit-llvm -o -
+// RUN: %clang_cc1 -no-opaque-pointers -triple i686-linux %s -emit-llvm -o - | FileCheck %s --check-prefix=X86_32
+// RUN: %clang_cc1 -no-opaque-pointers %s -emit-llvm -o -
 
 struct X {
   double D;
@@ -10,13 +10,13 @@ struct Y {
   struct X x;
 };
 
-struct Y bar();
+struct Y bar(void);
 
 void foo(struct Y *P) {
   *P = bar();
 }
 
-struct Y bar() {
+struct Y bar(void) {
   struct Y a;
   a.x.D = 0;
   return a;
