@@ -2,7 +2,7 @@
 // This test checks validity of inline assembly using curly brackets syntax
 // for extended inline asm.
 
-void test_curly_brackets() {
+void test_curly_brackets(void) {
     //CHECK:  %xmm1,%xmm0,%xmm1 {%k1}{z}
     asm("vpaddb\t %%xmm1,%%xmm0,%%xmm1 %{%%k1%}%{z%}\t":::);
 }
\ No newline at end of file
