@@ -3,7 +3,7 @@
 typedef void (*fptrs_t[4])(void);
 fptrs_t p __attribute__((noreturn));
 
-void __attribute__((noreturn)) f() {
+void __attribute__((noreturn)) f(void) {
   p[0]();
 }
 // CHECK: call void
