@@ -1,5 +1,5 @@
-// RUN: %clang_cc1 -disable-noundef-analysis           -triple x86_64-apple-darwin -emit-llvm %s -o - 2>&1 | FileCheck %s
-// RUN: %clang_cc1 -disable-noundef-analysis -DDYNAMIC -triple x86_64-apple-darwin -emit-llvm %s -o - 2>&1 | FileCheck %s
+// RUN: %clang_cc1 -no-opaque-pointers -no-enable-noundef-analysis           -triple x86_64-apple-darwin -emit-llvm %s -o - 2>&1 | FileCheck %s
+// RUN: %clang_cc1 -no-opaque-pointers -no-enable-noundef-analysis -DDYNAMIC -triple x86_64-apple-darwin -emit-llvm %s -o - 2>&1 | FileCheck %s
 
 #ifndef DYNAMIC
 #define OBJECT_SIZE_BUILTIN __builtin_object_size
@@ -21,38 +21,38 @@ char *gp;
 int gi, gj;
 
 // CHECK-LABEL: define{{.*}} void @test1
-void test1() {
+void test1(void) {
   // CHECK:     = call i8* @__strcpy_chk(i8* getelementptr inbounds ([63 x i8], [63 x i8]* @gbuf, i64 0, i64 4), i8* getelementptr inbounds ([9 x i8], [9 x i8]* @.str, i64 0, i64 0), i64 59)
   strcpy(&gbuf[4], "Hi there");
 }
 
 // CHECK-LABEL: define{{.*}} void @test2
-void test2() {
+void test2(void) {
   // CHECK:     = call i8* @__strcpy_chk(i8* getelementptr inbounds ([63 x i8], [63 x i8]* @gbuf, i64 0, i64 0), i8* getelementptr inbounds ([9 x i8], [9 x i8]* @.str, i64 0, i64 0), i64 63)
   strcpy(gbuf, "Hi there");
 }
 
 // CHECK-LABEL: define{{.*}} void @test3
-void test3() {
+void test3(void) {
   // CHECK:     = call i8* @__strcpy_chk(i8* getelementptr inbounds ([63 x i8], [63 x i8]* @gbuf, i64 1, i64 37), i8* getelementptr inbounds ([9 x i8], [9 x i8]* @.str, i64 0, i64 0), i64 0)
   strcpy(&gbuf[100], "Hi there");
 }
 
 // CHECK-LABEL: define{{.*}} void @test4
-void test4() {
+void test4(void) {
   // CHECK:     = call i8* @__strcpy_chk(i8* getelementptr inbounds ([63 x i8], [63 x i8]* @gbuf, i64 0, i64 -1), i8* getelementptr inbounds ([9 x i8], [9 x i8]* @.str, i64 0, i64 0), i64 0)
   strcpy((char*)(void*)&gbuf[-1], "Hi there");
 }
 
 // CHECK-LABEL: define{{.*}} void @test5
-void test5() {
+void test5(void) {
   // CHECK:     = load i8*, i8** @gp
   // CHECK-NEXT:= call i64 @llvm.objectsize.i64.p0i8(i8* %{{.*}}, i1 false, i1 true, i1
   strcpy(gp, "Hi there");
 }
 
 // CHECK-LABEL: define{{.*}} void @test6
-void test6() {
+void test6(void) {
   char buf[57];
 
   // CHECK:       = call i8* @__strcpy_chk(i8* %{{.*}}, i8* getelementptr inbounds ([9 x i8], [9 x i8]* @.str, i64 0, i64 0), i64 53)
@@ -60,7 +60,7 @@ void test6() {
 }
 
 // CHECK-LABEL: define{{.*}} void @test7
-void test7() {
+void test7(void) {
   int i;
   // Ensure we only evaluate the side-effect once.
   // CHECK:     = add
@@ -70,7 +70,7 @@ void test7() {
 }
 
 // CHECK-LABEL: define{{.*}} void @test8
-void test8() {
+void test8(void) {
   char *buf[50];
   // CHECK-NOT:   __strcpy_chk
   // CHECK:       = call i8* @__inline_strcpy_chk(i8* %{{.*}}, i8* getelementptr inbounds ([9 x i8], [9 x i8]* @.str, i64 0, i64 0))
@@ -78,7 +78,7 @@ void test8() {
 }
 
 // CHECK-LABEL: define{{.*}} void @test9
-void test9() {
+void test9(void) {
   // CHECK-NOT:   __strcpy_chk
   // CHECK:       = call i8* @__inline_strcpy_chk(i8* %{{.*}}, i8* getelementptr inbounds ([9 x i8], [9 x i8]* @.str, i64 0, i64 0))
   strcpy((char *)((++gi) + gj), "Hi there");
@@ -86,56 +86,56 @@ void test9() {
 
 // CHECK-LABEL: define{{.*}} void @test10
 char **p;
-void test10() {
+void test10(void) {
   // CHECK-NOT:   __strcpy_chk
   // CHECK:       = call i8* @__inline_strcpy_chk(i8* %{{.*}}, i8* getelementptr inbounds ([9 x i8], [9 x i8]* @.str, i64 0, i64 0))
   strcpy(*(++p), "Hi there");
 }
 
 // CHECK-LABEL: define{{.*}} void @test11
-void test11() {
+void test11(void) {
   // CHECK-NOT:   __strcpy_chk
   // CHECK:       = call i8* @__inline_strcpy_chk(i8* getelementptr inbounds ([63 x i8], [63 x i8]* @gbuf, i64 0, i64 0), i8* getelementptr inbounds ([9 x i8], [9 x i8]* @.str, i64 0, i64 0))
   strcpy(gp = gbuf, "Hi there");
 }
 
 // CHECK-LABEL: define{{.*}} void @test12
-void test12() {
+void test12(void) {
   // CHECK-NOT:   __strcpy_chk
   // CHECK:       = call i8* @__inline_strcpy_chk(i8* %{{.*}}, i8* getelementptr inbounds ([9 x i8], [9 x i8]* @.str, i64 0, i64 0))
   strcpy(++gp, "Hi there");
 }
 
 // CHECK-LABEL: define{{.*}} void @test13
-void test13() {
+void test13(void) {
   // CHECK-NOT:   __strcpy_chk
   // CHECK:       = call i8* @__inline_strcpy_chk(i8* %{{.*}}, i8* getelementptr inbounds ([9 x i8], [9 x i8]* @.str, i64 0, i64 0))
   strcpy(gp++, "Hi there");
 }
 
 // CHECK-LABEL: define{{.*}} void @test14
-void test14() {
+void test14(void) {
   // CHECK-NOT:   __strcpy_chk
   // CHECK:       = call i8* @__inline_strcpy_chk(i8* %{{.*}}, i8* getelementptr inbounds ([9 x i8], [9 x i8]* @.str, i64 0, i64 0))
   strcpy(--gp, "Hi there");
 }
 
 // CHECK-LABEL: define{{.*}} void @test15
-void test15() {
+void test15(void) {
   // CHECK-NOT:   __strcpy_chk
   // CHECK:       = call i8* @__inline_strcpy_chk(i8* %{{..*}}, i8* getelementptr inbounds ([9 x i8], [9 x i8]* @.str, i64 0, i64 0))
   strcpy(gp--, "Hi there");
 }
 
 // CHECK-LABEL: define{{.*}} void @test16
-void test16() {
+void test16(void) {
   // CHECK-NOT:   __strcpy_chk
   // CHECK:       = call i8* @__inline_strcpy_chk(i8* %{{.*}}, i8* getelementptr inbounds ([9 x i8], [9 x i8]* @.str, i64 0, i64 0))
   strcpy(gp += 1, "Hi there");
 }
 
 // CHECK-LABEL: @test17
-void test17() {
+void test17(void) {
   // CHECK: store i32 -1
   gi = OBJECT_SIZE_BUILTIN(gp++, 0);
   // CHECK: store i32 -1
@@ -155,7 +155,7 @@ unsigned test18(int cond) {
 }
 
 // CHECK-LABEL: @test19
-void test19() {
+void test19(void) {
   struct {
     int a, b;
   } foo;
@@ -180,7 +180,7 @@ void test19() {
 }
 
 // CHECK-LABEL: @test20
-void test20() {
+void test20(void) {
   struct { int t[10]; } t[10];
 
   // CHECK: store i32 380
@@ -194,7 +194,7 @@ void test20() {
 }
 
 // CHECK-LABEL: @test21
-void test21() {
+void test21(void) {
   struct { int t; } t;
 
   // CHECK: store i32 0
@@ -217,7 +217,7 @@ void test21() {
 }
 
 // CHECK-LABEL: @test22
-void test22() {
+void test22(void) {
   struct { int t[10]; } t[10];
 
   // CHECK: store i32 0
@@ -293,7 +293,7 @@ void test23(struct Test23Ty *p) {
 
 // PR24493 -- ICE if OBJECT_SIZE_BUILTIN called with NULL and (Type & 1) != 0
 // CHECK-LABEL: @test24
-void test24() {
+void test24(void) {
   // CHECK: call i64 @llvm.objectsize.i64.p0i8(i8* {{.*}}, i1 false, i1 true, i1
   gi = OBJECT_SIZE_BUILTIN((void*)0, 0);
   // CHECK: call i64 @llvm.objectsize.i64.p0i8(i8* {{.*}}, i1 false, i1 true, i1
@@ -307,7 +307,7 @@ void test24() {
 }
 
 // CHECK-LABEL: @test25
-void test25() {
+void test25(void) {
   // CHECK: call i64 @llvm.objectsize.i64.p0i8(i8* {{.*}}, i1 false, i1 true, i1
   gi = OBJECT_SIZE_BUILTIN((void*)0x1000, 0);
   // CHECK: call i64 @llvm.objectsize.i64.p0i8(i8* {{.*}}, i1 false, i1 true, i1
@@ -332,7 +332,7 @@ void test25() {
 }
 
 // CHECK-LABEL: @test26
-void test26() {
+void test26(void) {
   struct { int v[10]; } t[10];
 
   // CHECK: store i32 316
@@ -375,7 +375,7 @@ void test27(struct Test27IncompleteTy *t) {
 // The intent of this test is to ensure that OBJECT_SIZE_BUILTIN treats `&foo`
 // and `(T*)&foo` identically, when used as the pointer argument.
 // CHECK-LABEL: @test28
-void test28() {
+void test28(void) {
   struct { int v[10]; } t[10];
 
 #define addCasts(s) ((char*)((short*)(s)))
@@ -460,7 +460,7 @@ void test29(struct DynStructVar *dv, struct DynStruct0 *d0,
 }
 
 // CHECK-LABEL: @test30
-void test30() {
+void test30(void) {
   struct { struct DynStruct1 fst, snd; } *nested;
 
   // CHECK: call i64 @llvm.objectsize.i64.p0i8(i8* %{{.*}}, i1 false, i1 true, i1
@@ -502,7 +502,7 @@ void test30() {
 }
 
 // CHECK-LABEL: @test31
-void test31() {
+void test31(void) {
   // Miscellaneous 'writing off the end' detection tests
   struct DynStructVar *dsv;
   struct DynStruct0 *ds0;
@@ -526,7 +526,7 @@ void test31() {
 }
 
 // CHECK-LABEL: @PR30346
-void PR30346() {
+void PR30346(void) {
   struct sa_family_t {};
   struct sockaddr {
     struct sa_family_t sa_family;
@@ -546,7 +546,7 @@ void PR30346() {
 
 extern char incomplete_char_array[];
 // CHECK-LABEL: @incomplete_and_function_types
-int incomplete_and_function_types() {
+int incomplete_and_function_types(void) {
   // CHECK: call i64 @llvm.objectsize.i64.p0i8
   gi = OBJECT_SIZE_BUILTIN(incomplete_char_array, 0);
   // CHECK: call i64 @llvm.objectsize.i64.p0i8
@@ -558,7 +558,7 @@ int incomplete_and_function_types() {
 }
 
 // Flips between the pointer and lvalue evaluator a lot.
-void deeply_nested() {
+void deeply_nested(void) {
   struct {
     struct {
       struct {
