@@ -7,9 +7,9 @@
 void x(id y) {}
 void y(int a) {}
 
-extern id opaque_id();
+extern id opaque_id(void);
 
-void f() {
+void f(void) {
     __block int byref_int = 0;
     const id bar = (id) opaque_id();
     id baz = 0;
@@ -18,19 +18,19 @@ void f() {
     __block id bl_var1;
 
 // CHECK: Inline block variable layout: 0x0100, BL_STRONG:1, BL_OPERATOR:0
-    void (^b)() = ^{
+    void (^b)(void) = ^{
         x(bar);
     };    
 
 // CHECK: Inline block variable layout: 0x0210, BL_STRONG:2, BL_BYREF:1, BL_OPERATOR:0
-    void (^c)() = ^{
+    void (^c)(void) = ^{
         x(bar);
         x(baz);
         byref_int = 1;
     };    
 
 // CHECK: Inline block variable layout: 0x0230, BL_STRONG:2, BL_BYREF:3, BL_OPERATOR:0
-    void (^d)() = ^{
+    void (^d)(void) = ^{
         x(bar);
         x(baz);
         byref_int = 1;
@@ -40,7 +40,7 @@ void f() {
 
 // CHECK: Inline block variable layout: 0x0231, BL_STRONG:2, BL_BYREF:3, BL_WEAK:1, BL_OPERATOR:0
     __weak id wid;
-    id (^e)() = ^{
+    id (^e)(void) = ^{
         x(bar);
         x(baz);
         byref_int = 1;
@@ -51,7 +51,7 @@ void f() {
 
 // CHECK: Inline block variable layout: 0x0235, BL_STRONG:2, BL_BYREF:3, BL_WEAK:5, BL_OPERATOR:0
     __weak id wid1, wid2, wid3, wid4;
-    id (^f)() = ^{
+    id (^f)(void) = ^{
         x(bar);
         x(baz);
         byref_int = 1;
@@ -65,7 +65,7 @@ void f() {
     };
 
 // CHECK: Inline block variable layout: 0x035, BL_BYREF:3, BL_WEAK:5, BL_OPERATOR:0
-    id (^g)() = ^{
+    id (^g)(void) = ^{
         byref_int = 1;
         bl_var1 = 0;
         byref_bab = 0;
@@ -77,18 +77,18 @@ void f() {
     };
 
 // CHECK: Inline block variable layout: 0x01, BL_WEAK:1, BL_OPERATOR:0
-    id (^h)() = ^{
+    id (^h)(void) = ^{
         return wid;
     };
 
 // CHECK: Inline block variable layout: 0x020, BL_BYREF:2, BL_OPERATOR:0
-    void (^ii)() = ^{
+    void (^ii)(void) = ^{
        byref_int = 1;
        byref_bab = 0;
     };
 
 // CHECK: Inline block variable layout: 0x0102, BL_STRONG:1, BL_WEAK:2, BL_OPERATOR:0
-    void (^jj)() = ^{
+    void (^jj)(void) = ^{
       x(bar);
       x(wid1);
       x(wid2);
@@ -99,7 +99,7 @@ void f() {
 @class NSString;
 extern void NSLog(NSString *format, ...);
 typedef void (^dispatch_block_t)(void);
-int main() {
+int main(void) {
         __strong NSString *s1 = 0;
         __strong NSString *s2 = 0;
         __weak NSString *w1 = 0;
