@@ -1,4 +1,4 @@
-// RUN: %clang_cc1 -fblocks -triple x86_64-apple-darwin -fobjc-runtime=macosx-fragile-10.5 -emit-llvm -o - %s | FileCheck %s
+// RUN: %clang_cc1 -no-opaque-pointers -fblocks -triple x86_64-apple-darwin -fobjc-runtime=macosx-fragile-10.5 -emit-llvm -o - %s | FileCheck %s
 // rdar: // 8390455
 
 // CHECK: @ArrayRecurs = internal global
@@ -21,7 +21,7 @@ static  NSArray *(^ArrayRecurs)(NSArray *addresses, unsigned long level) = ^(NSA
 extern NSArray *address;
 extern unsigned long level;
 
-void FUNC()
+void FUNC(void)
 {
  ArrayRecurs(address, level);
 
@@ -50,7 +50,7 @@ void FUNC()
  }
 }
 
-void FUNC2() {
+void FUNC2(void) {
   static void (^const block1)(int) = ^(int a){
     if (a--)
       block1(a);
@@ -59,9 +59,9 @@ void FUNC2() {
 
 // CHECK-LABEL: define{{.*}} void @FUNC2(
 // CHECK: define internal void @_block_invoke{{.*}}(
-// CHECK: call void %{{.*}}(i8* bitcast ({ i8**, i32, i32, i8*, %struct.__block_descriptor* }* @__block_literal_global{{.*}} to i8*), i32 %{{.*}})
+// CHECK: call void %{{.*}}(i8* noundef bitcast ({ i8**, i32, i32, i8*, %struct.__block_descriptor* }* @__block_literal_global{{.*}} to i8*), i32 noundef %{{.*}})
 
-void FUNC1()
+void FUNC1(void)
 {
  static  NSArray *(^ArrayRecurs)(NSArray *addresses, unsigned long level) = ^(NSArray *addresses, unsigned long level) {
 
