@@ -1,16 +1,16 @@
 // RUN: rm -f %t.log
 // RUN: env RC_DEBUG_OPTIONS=1 \
 // RUN:     CC_LOG_DIAGNOSTICS=1 CC_LOG_DIAGNOSTICS_FILE=%t.log \
-// RUN: %clang -Wfoobar -no-canonical-prefixes -target x86_64-apple-darwin10 -fsyntax-only %s
+// RUN: %clang -Wfoobar --target=x86_64-apple-darwin10 -fsyntax-only %s
 // RUN: FileCheck %s < %t.log
 
-int f0() {}
+int f0(void) {}
 
 // CHECK: <dict>
 // CHECK:   <key>main-file</key>
 // CHECK:   <string>{{.*}}cc-log-diagnostics.c</string>
 // CHECK:   <key>dwarf-debug-flags</key>
-// CHECK:   <string>{{.*}}clang{{.*}}-fsyntax-only{{.*}}</string>
+// CHECK:   <string>{{.*}}-Wfoobar{{.*}}-fsyntax-only{{.*}}</string>
 // CHECK:   <key>diagnostics</key>
 // CHECK:   <array>
 // CHECK:     <dict>
@@ -27,7 +27,7 @@ int f0() {}
 // CHECK:       <key>line</key>
 // CHECK:       <integer>7</integer>
 // CHECK:       <key>column</key>
-// CHECK:       <integer>11</integer>
+// CHECK:       <integer>15</integer>
 // CHECK:       <key>message</key>
 // CHECK:       <string>non-void function does not return a value</string>
 // CHECK:     </dict>
