@@ -5,10 +5,10 @@
 int printf(const char * restrict, ...);
 typedef unsigned int NSUInteger;
 typedef int NSInteger;
-NSUInteger getNSUInteger();
-NSInteger getNSInteger();
+NSUInteger getNSUInteger(void);
+NSInteger getNSInteger(void);
 
-void test() {
+void test(void) {
   // For thumbv7-apple-ios8.0.0 the underlying type of ssize_t is long
   // and the underlying type of size_t is unsigned long.
 
