@@ -52,7 +52,7 @@
 
 int X = 0;
 
-int main() {
+int main(void) {
   int i;
   for (i = 0; i < 100; i++)
     X += i;
