@@ -12,6 +12,7 @@
 #include "clang/Tooling/Transformer/RangeSelector.h"
 #include "clang/Tooling/Transformer/RewriteRule.h"
 #include "clang/Tooling/Transformer/Stencil.h"
+#include "llvm/ADT/STLExtras.h"
 #include "llvm/Support/Errc.h"
 #include "llvm/Support/Error.h"
 #include "gmock/gmock.h"
@@ -26,16 +27,21 @@ using ::clang::transformer::applyFirst;
 using ::clang::transformer::before;
 using ::clang::transformer::cat;
 using ::clang::transformer::changeTo;
+using ::clang::transformer::editList;
 using ::clang::transformer::makeRule;
 using ::clang::transformer::member;
 using ::clang::transformer::name;
 using ::clang::transformer::node;
+using ::clang::transformer::noEdits;
 using ::clang::transformer::remove;
 using ::clang::transformer::rewriteDescendants;
 using ::clang::transformer::RewriteRule;
+using ::clang::transformer::RewriteRuleWith;
 using ::clang::transformer::statement;
 using ::testing::ElementsAre;
 using ::testing::IsEmpty;
+using ::testing::ResultOf;
+using ::testing::UnorderedElementsAre;
 
 constexpr char KHeaderContents[] = R"cc(
   struct string {
@@ -77,7 +83,7 @@ static std::string format(StringRef Code) {
 }
 
 static void compareSnippets(StringRef Expected,
-                     const llvm::Optional<std::string> &MaybeActual) {
+                            const llvm::Optional<std::string> &MaybeActual) {
   ASSERT_TRUE(MaybeActual) << "Rewrite failed. Expecting: " << Expected;
   auto Actual = *MaybeActual;
   std::string HL = "#include \"header.h\"\n";
@@ -120,12 +126,13 @@ protected:
     return *ChangedCode;
   }
 
-  Transformer::ChangeConsumer consumer() {
-    return [this](Expected<AtomicChange> C) {
+  Transformer::ChangeSetConsumer consumer() {
+    return [this](Expected<MutableArrayRef<AtomicChange>> C) {
       if (C) {
-        Changes.push_back(std::move(*C));
+        Changes.insert(Changes.end(), std::make_move_iterator(C->begin()),
+                       std::make_move_iterator(C->end()));
       } else {
-        // FIXME: stash this error rather then printing.
+        // FIXME: stash this error rather than printing.
         llvm::errs() << "Error generating changes: "
                      << llvm::toString(C.takeError()) << "\n";
         ++ErrorCount;
@@ -133,27 +140,58 @@ protected:
     };
   }
 
-  template <typename R>
-  void testRule(R Rule, StringRef Input, StringRef Expected) {
+  auto consumerWithStringMetadata() {
+    return [this](Expected<TransformerResult<std::string>> C) {
+      if (C) {
+        Changes.insert(Changes.end(),
+                       std::make_move_iterator(C->Changes.begin()),
+                       std::make_move_iterator(C->Changes.end()));
+        StringMetadata.push_back(std::move(C->Metadata));
+      } else {
+        // FIXME: stash this error rather than printing.
+        llvm::errs() << "Error generating changes: "
+                     << llvm::toString(C.takeError()) << "\n";
+        ++ErrorCount;
+      }
+    };
+  }
+
+  void testRule(RewriteRule Rule, StringRef Input, StringRef Expected) {
     Transformers.push_back(
         std::make_unique<Transformer>(std::move(Rule), consumer()));
     Transformers.back()->registerMatchers(&MatchFinder);
     compareSnippets(Expected, rewrite(Input));
   }
 
-  template <typename R> void testRuleFailure(R Rule, StringRef Input) {
+  void testRule(RewriteRuleWith<std::string> Rule, StringRef Input,
+                StringRef Expected) {
+    Transformers.push_back(std::make_unique<Transformer>(
+        std::move(Rule), consumerWithStringMetadata()));
+    Transformers.back()->registerMatchers(&MatchFinder);
+    compareSnippets(Expected, rewrite(Input));
+  }
+
+  void testRuleFailure(RewriteRule Rule, StringRef Input) {
     Transformers.push_back(
         std::make_unique<Transformer>(std::move(Rule), consumer()));
     Transformers.back()->registerMatchers(&MatchFinder);
     ASSERT_FALSE(rewrite(Input)) << "Expected failure to rewrite code";
   }
 
+  void testRuleFailure(RewriteRuleWith<std::string> Rule, StringRef Input) {
+    Transformers.push_back(std::make_unique<Transformer>(
+        std::move(Rule), consumerWithStringMetadata()));
+    Transformers.back()->registerMatchers(&MatchFinder);
+    ASSERT_FALSE(rewrite(Input)) << "Expected failure to rewrite code";
+  }
+
   // Transformers are referenced by MatchFinder.
   std::vector<std::unique_ptr<Transformer>> Transformers;
   clang::ast_matchers::MatchFinder MatchFinder;
   // Records whether any errors occurred in individual changes.
   int ErrorCount = 0;
   AtomicChanges Changes;
+  std::vector<std::string> StringMetadata;
 
 private:
   FileContentMappings FileContents = {{"header.h", ""}};
@@ -165,7 +203,7 @@ protected:
 };
 
 // Given string s, change strlen($s.c_str()) to REPLACED.
-static RewriteRule ruleStrlenSize() {
+static RewriteRuleWith<std::string> ruleStrlenSize() {
   StringRef StringExpr = "strexpr";
   auto StringType = namedDecl(hasAnyName("::basic_string", "::string"));
   auto R = makeRule(
@@ -799,7 +837,6 @@ TEST_F(TransformerTest, MultiChange) {
 }
 
 TEST_F(TransformerTest, EditList) {
-  using clang::transformer::editList;
   std::string Input = R"cc(
     void foo() {
       if (10 > 1.0)
@@ -827,7 +864,6 @@ TEST_F(TransformerTest, EditList) {
 }
 
 TEST_F(TransformerTest, Flatten) {
-  using clang::transformer::editList;
   std::string Input = R"cc(
     void foo() {
       if (10 > 1.0)
@@ -884,12 +920,12 @@ TEST_F(TransformerTest, FlattenWithMixedArgs) {
 
 TEST_F(TransformerTest, OrderedRuleUnrelated) {
   StringRef Flag = "flag";
-  RewriteRule FlagRule = makeRule(
+  RewriteRuleWith<std::string> FlagRule = makeRule(
       cxxMemberCallExpr(on(expr(hasType(cxxRecordDecl(
                                     hasName("proto::ProtoCommandLineFlag"))))
                                .bind(Flag)),
                         unless(callee(cxxMethodDecl(hasName("GetProto"))))),
-      changeTo(node(std::string(Flag)), cat("PROTO")));
+      changeTo(node(std::string(Flag)), cat("PROTO")), cat(""));
 
   std::string Input = R"cc(
     proto::ProtoCommandLineFlag flag;
@@ -1230,31 +1266,29 @@ void testIt()
   auto RewriteOutput =
       CodePrefix + RangeLoop + LoopBody + RangeLoop + LoopBody + CodeSuffix;
 
-    auto MatchedLoop = forStmt(
-        has(declStmt(
-            hasSingleDecl(varDecl(hasInitializer(integerLiteral(equals(0))))
-                              .bind("loopVar")))),
-        has(binaryOperator(hasOperatorName("!="),
-                           hasLHS(ignoringImplicit(declRefExpr(
-                               to(varDecl(equalsBoundNode("loopVar")))))),
-                           hasRHS(expr().bind("upperBoundExpr")))),
-        has(unaryOperator(hasOperatorName("++"),
-                          hasUnaryOperand(declRefExpr(
-                              to(varDecl(equalsBoundNode("loopVar"))))))
-                .bind("incrementOp")));
+  auto MatchedLoop = forStmt(
+      has(declStmt(hasSingleDecl(
+          varDecl(hasInitializer(integerLiteral(equals(0)))).bind("loopVar")))),
+      has(binaryOperator(hasOperatorName("!="),
+                         hasLHS(ignoringImplicit(declRefExpr(
+                             to(varDecl(equalsBoundNode("loopVar")))))),
+                         hasRHS(expr().bind("upperBoundExpr")))),
+      has(unaryOperator(hasOperatorName("++"),
+                        hasUnaryOperand(declRefExpr(
+                            to(varDecl(equalsBoundNode("loopVar"))))))
+              .bind("incrementOp")));
 
-    auto RewriteRule =
-        changeTo(transformer::enclose(node("loopVar"), node("incrementOp")),
-                 cat("auto ", name("loopVar"), " : boost::irange(",
-                     node("upperBoundExpr"), ")"));
+  auto RewriteRule =
+      changeTo(transformer::enclose(node("loopVar"), node("incrementOp")),
+               cat("auto ", name("loopVar"), " : boost::irange(",
+                   node("upperBoundExpr"), ")"));
 
-    testRule(makeRule(traverse(TK_IgnoreUnlessSpelledInSource, MatchedLoop),
-                      RewriteRule),
-             RewriteInput, RewriteOutput);
-
-    testRuleFailure(makeRule(traverse(TK_AsIs, MatchedLoop), RewriteRule),
-                    RewriteInput);
+  testRule(makeRule(traverse(TK_IgnoreUnlessSpelledInSource, MatchedLoop),
+                    RewriteRule),
+           RewriteInput, RewriteOutput);
 
+  testRuleFailure(makeRule(traverse(TK_AsIs, MatchedLoop), RewriteRule),
+                  RewriteInput);
 }
 
 TEST_F(TransformerTest, ImplicitNodes_ForStmt2) {
@@ -1303,31 +1337,29 @@ void testIt()
 
   auto RewriteOutput =
       CodePrefix + RangeLoop + LoopBody + RangeLoop + LoopBody + CodeSuffix;
-    auto MatchedLoop = forStmt(
-        hasLoopInit(declStmt(
-            hasSingleDecl(varDecl(hasInitializer(integerLiteral(equals(0))))
-                              .bind("loopVar")))),
-        hasCondition(binaryOperator(hasOperatorName("!="),
-                                    hasLHS(ignoringImplicit(declRefExpr(to(
-                                        varDecl(equalsBoundNode("loopVar")))))),
-                                    hasRHS(expr().bind("upperBoundExpr")))),
-        hasIncrement(unaryOperator(hasOperatorName("++"),
-                                   hasUnaryOperand(declRefExpr(to(
-                                       varDecl(equalsBoundNode("loopVar"))))))
-                         .bind("incrementOp")));
+  auto MatchedLoop = forStmt(
+      hasLoopInit(declStmt(hasSingleDecl(
+          varDecl(hasInitializer(integerLiteral(equals(0)))).bind("loopVar")))),
+      hasCondition(binaryOperator(hasOperatorName("!="),
+                                  hasLHS(ignoringImplicit(declRefExpr(to(
+                                      varDecl(equalsBoundNode("loopVar")))))),
+                                  hasRHS(expr().bind("upperBoundExpr")))),
+      hasIncrement(unaryOperator(hasOperatorName("++"),
+                                 hasUnaryOperand(declRefExpr(
+                                     to(varDecl(equalsBoundNode("loopVar"))))))
+                       .bind("incrementOp")));
 
-    auto RewriteRule =
-        changeTo(transformer::enclose(node("loopVar"), node("incrementOp")),
-                 cat("auto ", name("loopVar"), " : boost::irange(",
-                     node("upperBoundExpr"), ")"));
+  auto RewriteRule =
+      changeTo(transformer::enclose(node("loopVar"), node("incrementOp")),
+               cat("auto ", name("loopVar"), " : boost::irange(",
+                   node("upperBoundExpr"), ")"));
 
-    testRule(makeRule(traverse(TK_IgnoreUnlessSpelledInSource, MatchedLoop),
-                      RewriteRule),
-             RewriteInput, RewriteOutput);
-
-    testRuleFailure(makeRule(traverse(TK_AsIs, MatchedLoop), RewriteRule),
-                    RewriteInput);
+  testRule(makeRule(traverse(TK_IgnoreUnlessSpelledInSource, MatchedLoop),
+                    RewriteRule),
+           RewriteInput, RewriteOutput);
 
+  testRuleFailure(makeRule(traverse(TK_AsIs, MatchedLoop), RewriteRule),
+                  RewriteInput);
 }
 
 TEST_F(TransformerTest, TemplateInstantiation) {
@@ -1586,10 +1618,9 @@ TEST_F(TransformerTest, MultipleFiles) {
       "clang-tool", std::make_shared<PCHContainerOperations>(),
       {{"input.h", Header}}));
 
-  std::sort(Changes.begin(), Changes.end(),
-            [](const AtomicChange &L, const AtomicChange &R) {
-              return L.getFilePath() < R.getFilePath();
-            });
+  llvm::sort(Changes, [](const AtomicChange &L, const AtomicChange &R) {
+    return L.getFilePath() < R.getFilePath();
+  });
 
   ASSERT_EQ(Changes[0].getFilePath(), "./input.h");
   EXPECT_THAT(Changes[0].getInsertedHeaders(), IsEmpty());
@@ -1638,4 +1669,81 @@ TEST_F(TransformerTest, AddIncludeMultipleFiles) {
       << "Could not update code: " << llvm::toString(UpdatedCode.takeError());
   EXPECT_EQ(format(*UpdatedCode), format(Header));
 }
+
+// A single change set can span multiple files.
+TEST_F(TransformerTest, MultiFileEdit) {
+  // NB: The fixture is unused for this test, but kept for the test suite name.
+  std::string Header = R"cc(void Func(int id);)cc";
+  std::string Source = R"cc(#include "input.h"
+                            void Caller() {
+                              int id = 0;
+                              Func(id);
+                            })cc";
+  int ErrorCount = 0;
+  std::vector<AtomicChanges> ChangeSets;
+  clang::ast_matchers::MatchFinder MatchFinder;
+  Transformer T(
+      makeRule(callExpr(callee(functionDecl(hasName("Func"))),
+                        forEachArgumentWithParam(expr().bind("arg"),
+                                                 parmVarDecl().bind("param"))),
+               {changeTo(node("arg"), cat("ARG")),
+                changeTo(node("param"), cat("PARAM"))}),
+      [&](Expected<MutableArrayRef<AtomicChange>> Changes) {
+        if (Changes)
+          ChangeSets.push_back(AtomicChanges(Changes->begin(), Changes->end()));
+        else
+          ++ErrorCount;
+      });
+  T.registerMatchers(&MatchFinder);
+  auto Factory = newFrontendActionFactory(&MatchFinder);
+  EXPECT_TRUE(runToolOnCodeWithArgs(
+      Factory->create(), Source, std::vector<std::string>(), "input.cc",
+      "clang-tool", std::make_shared<PCHContainerOperations>(),
+      {{"input.h", Header}}));
+
+  EXPECT_EQ(ErrorCount, 0);
+  EXPECT_THAT(
+      ChangeSets,
+      UnorderedElementsAre(UnorderedElementsAre(
+          ResultOf([](const AtomicChange &C) { return C.getFilePath(); },
+                   "input.cc"),
+          ResultOf([](const AtomicChange &C) { return C.getFilePath(); },
+                   "./input.h"))));
+}
+
+TEST_F(TransformerTest, GeneratesMetadata) {
+  std::string Input = R"cc(int target = 0;)cc";
+  std::string Expected = R"cc(REPLACE)cc";
+  RewriteRuleWith<std::string> Rule = makeRule(
+      varDecl(hasName("target")), changeTo(cat("REPLACE")), cat("METADATA"));
+  testRule(std::move(Rule), Input, Expected);
+  EXPECT_EQ(ErrorCount, 0);
+  EXPECT_THAT(StringMetadata, UnorderedElementsAre("METADATA"));
+}
+
+TEST_F(TransformerTest, GeneratesMetadataWithNoEdits) {
+  std::string Input = R"cc(int target = 0;)cc";
+  RewriteRuleWith<std::string> Rule = makeRule(
+      varDecl(hasName("target")).bind("var"), noEdits(), cat("METADATA"));
+  testRule(std::move(Rule), Input, Input);
+  EXPECT_EQ(ErrorCount, 0);
+  EXPECT_THAT(StringMetadata, UnorderedElementsAre("METADATA"));
+}
+
+TEST_F(TransformerTest, PropagateMetadataErrors) {
+  class AlwaysFail : public transformer::MatchComputation<std::string> {
+    llvm::Error eval(const ast_matchers::MatchFinder::MatchResult &,
+                     std::string *) const override {
+      return llvm::createStringError(llvm::errc::invalid_argument, "ERROR");
+    }
+    std::string toString() const override { return "AlwaysFail"; }
+  };
+  std::string Input = R"cc(int target = 0;)cc";
+  RewriteRuleWith<std::string> Rule = makeRule<std::string>(
+      varDecl(hasName("target")).bind("var"), changeTo(cat("REPLACE")),
+      std::make_shared<AlwaysFail>());
+  testRuleFailure(std::move(Rule), Input);
+  EXPECT_EQ(ErrorCount, 1);
+}
+
 } // namespace
