@@ -11,15 +11,22 @@
 
 #define LIBC_INLINE_ASM __asm__ __volatile__
 
+#ifndef likely
 #define likely(x) __builtin_expect(!!(x), 1)
+#endif
+#ifndef unlikely
 #define unlikely(x) __builtin_expect(x, 0)
+#endif
+#ifndef UNUSED
 #define UNUSED __attribute__((unused))
+#endif
 
 #ifndef LLVM_LIBC_FUNCTION_ATTR
 #define LLVM_LIBC_FUNCTION_ATTR
 #endif
 
-#ifdef LLVM_LIBC_PUBLIC_PACKAGING
+// MacOS needs to be excluded because it does not support aliasing.
+#if defined(LLVM_LIBC_PUBLIC_PACKAGING) && (!defined(__APPLE__))
 #define LLVM_LIBC_FUNCTION(type, name, arglist)                                \
   LLVM_LIBC_FUNCTION_ATTR decltype(__llvm_libc::name)                          \
       __##name##_impl__ __asm__(#name);                                        \
