@@ -7,13 +7,13 @@
 //===----------------------------------------------------------------------===//
 
 #include "src/math/nearbyint.h"
+#include "src/__support/FPUtil/NearestIntegerOperations.h"
 #include "src/__support/common.h"
-#include "utils/FPUtil/NearestIntegerOperations.h"
 
 namespace __llvm_libc {
 
 LLVM_LIBC_FUNCTION(double, nearbyint, (double x)) {
-  return fputil::roundUsingCurrentRoundingMode(x);
+  return fputil::round_using_current_rounding_mode(x);
 }
 
 } // namespace __llvm_libc
