@@ -10,8 +10,8 @@
 // strongly control the options this file is compiled with. __restore_rt cannot
 // make any stack allocations so we must ensure this.
 
-#include "config/linux/syscall.h"
 #include "include/sys/syscall.h"
+#include "src/__support/OSUtil/syscall.h"
 
 namespace __llvm_libc {
 
