@@ -8,10 +8,11 @@
 
 #include "src/sys/mman/munmap.h"
 
-#include "config/linux/syscall.h" // For internal syscall function.
-#include "include/sys/syscall.h"  // For syscall numbers.
+#include "src/__support/OSUtil/syscall.h" // For internal syscall function.
 #include "src/__support/common.h"
-#include "src/errno/llvmlibc_errno.h"
+
+#include <errno.h>
+#include <sys/syscall.h>          // For syscall numbers.
 
 namespace __llvm_libc {
 
@@ -24,7 +25,7 @@ LLVM_LIBC_FUNCTION(int, munmap, (void *addr, size_t size)) {
   // A negative return value indicates an error with the magnitude of the
   // value being the error code.
   if (ret_val < 0) {
-    llvmlibc_errno = -ret_val;
+    errno = -ret_val;
     return -1;
   }
 
