@@ -8,31 +8,31 @@
 
 #include "ILogbTest.h"
 
+#include "src/__support/FPUtil/FPBits.h"
+#include "src/__support/FPUtil/ManipulationFunctions.h"
 #include "src/math/ilogbl.h"
-#include "utils/FPUtil/FPBits.h"
-#include "utils/FPUtil/ManipulationFunctions.h"
-#include "utils/FPUtil/TestHelpers.h"
+#include "utils/UnitTest/FPMatcher.h"
 #include "utils/UnitTest/Test.h"
 #include <math.h>
 
 using RunContext = __llvm_libc::testing::RunContext;
 
 TEST_F(LlvmLibcILogbTest, SpecialNumbers_ilogbl) {
-  testSpecialNumbers<long double>(&__llvm_libc::ilogbl);
+  test_special_numbers<long double>(&__llvm_libc::ilogbl);
 }
 
 TEST_F(LlvmLibcILogbTest, PowersOfTwo_ilogbl) {
-  testPowersOfTwo<long double>(&__llvm_libc::ilogbl);
+  test_powers_of_two<long double>(&__llvm_libc::ilogbl);
 }
 
 TEST_F(LlvmLibcILogbTest, SomeIntegers_ilogbl) {
-  testSomeIntegers<long double>(&__llvm_libc::ilogbl);
+  test_some_integers<long double>(&__llvm_libc::ilogbl);
 }
 
 TEST_F(LlvmLibcILogbTest, SubnormalRange_ilogbl) {
-  testSubnormalRange<long double>(&__llvm_libc::ilogbl);
+  test_subnormal_range<long double>(&__llvm_libc::ilogbl);
 }
 
 TEST_F(LlvmLibcILogbTest, NormalRange_ilogbl) {
-  testNormalRange<long double>(&__llvm_libc::ilogbl);
+  test_normal_range<long double>(&__llvm_libc::ilogbl);
 }
