@@ -17,9 +17,7 @@ namespace lldb_private {
 class StackID {
 public:
   // Constructors and Destructors
-  StackID()
-
-  {}
+  StackID() = default;
 
   explicit StackID(lldb::addr_t pc, lldb::addr_t cfa,
                    SymbolContextScope *symbol_scope)
