@@ -23,12 +23,21 @@ class PExpectTest(TestBase):
     def expect_prompt(self):
         self.child.expect_exact(self.PROMPT)
 
-    def launch(self, executable=None, extra_args=None, timeout=60, dimensions=None):
+    def launch(self, executable=None, extra_args=None, timeout=60,
+               dimensions=None, run_under=None, post_spawn=None,
+               use_colors=False):
         logfile = getattr(sys.stdout, 'buffer',
                             sys.stdout) if self.TraceOn() else None
 
-        args = ['--no-lldbinit', '--no-use-colors']
+        args = []
+        if run_under is not None:
+            args += run_under
+        args += [lldbtest_config.lldbExec, '--no-lldbinit']
+        if not use_colors:
+            args.append('--no-use-colors')
         for cmd in self.setUpCommands():
+            if "use-color false" in cmd and use_colors:
+                continue
             args += ['-O', cmd]
         if executable is not None:
             args += ['--file', executable]
@@ -41,10 +50,17 @@ class PExpectTest(TestBase):
 
         import pexpect
         self.child = pexpect.spawn(
-                lldbtest_config.lldbExec, args=args, logfile=logfile,
+                args[0], args=args[1:], logfile=logfile,
                 timeout=timeout, dimensions=dimensions, env=env)
+        self.child.ptyproc.delayafterclose = timeout/10
+        self.child.ptyproc.delayafterterminate = timeout/10
+
+        if post_spawn is not None:
+            post_spawn()
         self.expect_prompt()
         for cmd in self.setUpCommands():
+            if "use-color false" in cmd and use_colors:
+                continue
             self.child.expect_exact(cmd)
             self.expect_prompt()
         if executable is not None:
