@@ -668,7 +668,7 @@ bool EmulateInstructionMIPS::GetRegisterInfo(RegisterKind reg_kind,
 }
 
 EmulateInstructionMIPS::MipsOpcode *
-EmulateInstructionMIPS::GetOpcodeForInstruction(const char *op_name) {
+EmulateInstructionMIPS::GetOpcodeForInstruction(llvm::StringRef name) {
   static EmulateInstructionMIPS::MipsOpcode g_opcodes[] = {
       // Prologue/Epilogue instructions
       {"ADDiu", &EmulateInstructionMIPS::Emulate_ADDiu,
@@ -954,13 +954,10 @@ EmulateInstructionMIPS::GetOpcodeForInstruction(const char *op_name) {
       {"JALRS_MM", &EmulateInstructionMIPS::Emulate_JALRS, "JALRS rt, rs"},
   };
 
-  static const size_t k_num_mips_opcodes = llvm::array_lengthof(g_opcodes);
-
-  for (size_t i = 0; i < k_num_mips_opcodes; ++i) {
-    if (!strcasecmp(g_opcodes[i].op_name, op_name))
-      return &g_opcodes[i];
+  for (MipsOpcode &opcode : g_opcodes) {
+    if (name.equals_insensitive(opcode.op_name))
+      return &opcode;
   }
-
   return nullptr;
 }
 
@@ -1342,7 +1339,7 @@ bool EmulateInstructionMIPS::Emulate_SUBU_ADDU(llvm::MCInst &insn) {
   bool success = false;
   uint64_t result;
   uint8_t src, dst, rt;
-  const char *op_name = m_insn_info->getName(insn.getOpcode()).data();
+  llvm::StringRef op_name = m_insn_info->getName(insn.getOpcode());
 
   dst = m_reg_info->getEncodingValue(insn.getOperand(0).getReg());
   src = m_reg_info->getEncodingValue(insn.getOperand(1).getReg());
@@ -1363,7 +1360,7 @@ bool EmulateInstructionMIPS::Emulate_SUBU_ADDU(llvm::MCInst &insn) {
     if (!success)
       return false;
 
-    if (!strcasecmp(op_name, "SUBU"))
+    if (op_name.equals_insensitive("SUBU"))
       result = src_opd_val - rt_opd_val;
     else
       result = src_opd_val + rt_opd_val;
@@ -1396,7 +1393,7 @@ bool EmulateInstructionMIPS::Emulate_SUBU_ADDU(llvm::MCInst &insn) {
 
     Context context;
 
-    if (!strcasecmp(op_name, "SUBU"))
+    if (op_name.equals_insensitive("SUBU"))
       result = src_opd_val - rt_opd_val;
     else
       result = src_opd_val + rt_opd_val;
@@ -1794,7 +1791,7 @@ bool EmulateInstructionMIPS::Emulate_BXX_3ops(llvm::MCInst &insn) {
   bool success = false;
   uint32_t rs, rt;
   int32_t offset, pc, target = 0, rs_val, rt_val;
-  const char *op_name = m_insn_info->getName(insn.getOpcode()).data();
+  llvm::StringRef op_name = m_insn_info->getName(insn.getOpcode());
 
   rs = m_reg_info->getEncodingValue(insn.getOperand(0).getReg());
   rt = m_reg_info->getEncodingValue(insn.getOperand(1).getReg());
@@ -1814,12 +1811,13 @@ bool EmulateInstructionMIPS::Emulate_BXX_3ops(llvm::MCInst &insn) {
   if (!success)
     return false;
 
-  if (!strcasecmp(op_name, "BEQ") || !strcasecmp(op_name, "BEQL")) {
+  if (op_name.equals_insensitive("BEQ") || op_name.equals_insensitive("BEQL")) {
     if (rs_val == rt_val)
       target = pc + offset;
     else
       target = pc + 8;
-  } else if (!strcasecmp(op_name, "BNE") || !strcasecmp(op_name, "BNEL")) {
+  } else if (op_name.equals_insensitive("BNE") ||
+             op_name.equals_insensitive("BNEL")) {
     if (rs_val != rt_val)
       target = pc + offset;
     else
@@ -1843,7 +1841,7 @@ bool EmulateInstructionMIPS::Emulate_BXX_3ops_C(llvm::MCInst &insn) {
   bool success = false;
   uint32_t rs, rt;
   int32_t offset, pc, target = 0, rs_val, rt_val;
-  const char *op_name = m_insn_info->getName(insn.getOpcode()).data();
+  llvm::StringRef op_name = m_insn_info->getName(insn.getOpcode());
   uint32_t current_inst_size = m_insn_info->get(insn.getOpcode()).getSize();
 
   rs = m_reg_info->getEncodingValue(insn.getOperand(0).getReg());
@@ -1864,42 +1862,42 @@ bool EmulateInstructionMIPS::Emulate_BXX_3ops_C(llvm::MCInst &insn) {
   if (!success)
     return false;
 
-  if (!strcasecmp(op_name, "BEQC")) {
+  if (op_name.equals_insensitive("BEQC")) {
     if (rs_val == rt_val)
       target = pc + offset;
     else
       target = pc + 4;
-  } else if (!strcasecmp(op_name, "BNEC")) {
+  } else if (op_name.equals_insensitive("BNEC")) {
     if (rs_val != rt_val)
       target = pc + offset;
     else
       target = pc + 4;
-  } else if (!strcasecmp(op_name, "BLTC")) {
+  } else if (op_name.equals_insensitive("BLTC")) {
     if (rs_val < rt_val)
       target = pc + offset;
     else
       target = pc + 4;
-  } else if (!strcasecmp(op_name, "BGEC")) {
+  } else if (op_name.equals_insensitive("BGEC")) {
     if (rs_val >= rt_val)
       target = pc + offset;
     else
       target = pc + 4;
-  } else if (!strcasecmp(op_name, "BLTUC")) {
+  } else if (op_name.equals_insensitive("BLTUC")) {
     if (rs_val < rt_val)
       target = pc + offset;
     else
       target = pc + 4;
-  } else if (!strcasecmp(op_name, "BGEUC")) {
+  } else if (op_name.equals_insensitive("BGEUC")) {
     if ((uint32_t)rs_val >= (uint32_t)rt_val)
       target = pc + offset;
     else
       target = pc + 4;
-  } else if (!strcasecmp(op_name, "BOVC")) {
+  } else if (op_name.equals_insensitive("BOVC")) {
     if (IsAdd64bitOverflow(rs_val, rt_val))
       target = pc + offset;
     else
       target = pc + 4;
-  } else if (!strcasecmp(op_name, "BNVC")) {
+  } else if (op_name.equals_insensitive("BNVC")) {
     if (!IsAdd64bitOverflow(rs_val, rt_val))
       target = pc + offset;
     else
@@ -1923,7 +1921,7 @@ bool EmulateInstructionMIPS::Emulate_Bcond_Link_C(llvm::MCInst &insn) {
   uint32_t rs;
   int32_t offset, pc, target = 0;
   int32_t rs_val;
-  const char *op_name = m_insn_info->getName(insn.getOpcode()).data();
+  llvm::StringRef op_name = m_insn_info->getName(insn.getOpcode());
 
   rs = m_reg_info->getEncodingValue(insn.getOperand(0).getReg());
   offset = insn.getOperand(1).getImm();
@@ -1937,32 +1935,32 @@ bool EmulateInstructionMIPS::Emulate_Bcond_Link_C(llvm::MCInst &insn) {
   if (!success)
     return false;
 
-  if (!strcasecmp(op_name, "BLEZALC")) {
+  if (op_name.equals_insensitive("BLEZALC")) {
     if (rs_val <= 0)
       target = pc + offset;
     else
       target = pc + 4;
-  } else if (!strcasecmp(op_name, "BGEZALC")) {
+  } else if (op_name.equals_insensitive("BGEZALC")) {
     if (rs_val >= 0)
       target = pc + offset;
     else
       target = pc + 4;
-  } else if (!strcasecmp(op_name, "BLTZALC")) {
+  } else if (op_name.equals_insensitive("BLTZALC")) {
     if (rs_val < 0)
       target = pc + offset;
     else
       target = pc + 4;
-  } else if (!strcasecmp(op_name, "BGTZALC")) {
+  } else if (op_name.equals_insensitive("BGTZALC")) {
     if (rs_val > 0)
       target = pc + offset;
     else
       target = pc + 4;
-  } else if (!strcasecmp(op_name, "BEQZALC")) {
+  } else if (op_name.equals_insensitive("BEQZALC")) {
     if (rs_val == 0)
       target = pc + offset;
     else
       target = pc + 4;
-  } else if (!strcasecmp(op_name, "BNEZALC")) {
+  } else if (op_name.equals_insensitive("BNEZALC")) {
     if (rs_val != 0)
       target = pc + offset;
     else
@@ -1992,7 +1990,7 @@ bool EmulateInstructionMIPS::Emulate_Bcond_Link(llvm::MCInst &insn) {
   uint32_t rs;
   int32_t offset, pc, target = 0;
   int32_t rs_val;
-  const char *op_name = m_insn_info->getName(insn.getOpcode()).data();
+  llvm::StringRef op_name = m_insn_info->getName(insn.getOpcode());
 
   rs = m_reg_info->getEncodingValue(insn.getOperand(0).getReg());
   offset = insn.getOperand(1).getImm();
@@ -2006,13 +2004,14 @@ bool EmulateInstructionMIPS::Emulate_Bcond_Link(llvm::MCInst &insn) {
   if (!success)
     return false;
 
-  if (!strcasecmp(op_name, "BLTZAL") || !strcasecmp(op_name, "BLTZALL")) {
+  if (op_name.equals_insensitive("BLTZAL") ||
+      op_name.equals_insensitive("BLTZALL")) {
     if ((int32_t)rs_val < 0)
       target = pc + offset;
     else
       target = pc + 8;
-  } else if (!strcasecmp(op_name, "BGEZAL") ||
-             !strcasecmp(op_name, "BGEZALL")) {
+  } else if (op_name.equals_insensitive("BGEZAL") ||
+             op_name.equals_insensitive("BGEZALL")) {
     if ((int32_t)rs_val >= 0)
       target = pc + offset;
     else
@@ -2042,7 +2041,7 @@ bool EmulateInstructionMIPS::Emulate_BXX_2ops(llvm::MCInst &insn) {
   uint32_t rs;
   int32_t offset, pc, target = 0;
   int32_t rs_val;
-  const char *op_name = m_insn_info->getName(insn.getOpcode()).data();
+  llvm::StringRef op_name = m_insn_info->getName(insn.getOpcode());
 
   rs = m_reg_info->getEncodingValue(insn.getOperand(0).getReg());
   offset = insn.getOperand(1).getImm();
@@ -2056,22 +2055,26 @@ bool EmulateInstructionMIPS::Emulate_BXX_2ops(llvm::MCInst &insn) {
   if (!success)
     return false;
 
-  if (!strcasecmp(op_name, "BLTZL") || !strcasecmp(op_name, "BLTZ")) {
+  if (op_name.equals_insensitive("BLTZL") ||
+      op_name.equals_insensitive("BLTZ")) {
     if (rs_val < 0)
       target = pc + offset;
     else
       target = pc + 8;
-  } else if (!strcasecmp(op_name, "BGEZL") || !strcasecmp(op_name, "BGEZ")) {
+  } else if (op_name.equals_insensitive("BGEZL") ||
+             op_name.equals_insensitive("BGEZ")) {
     if (rs_val >= 0)
       target = pc + offset;
     else
       target = pc + 8;
-  } else if (!strcasecmp(op_name, "BGTZL") || !strcasecmp(op_name, "BGTZ")) {
+  } else if (op_name.equals_insensitive("BGTZL") ||
+             op_name.equals_insensitive("BGTZ")) {
     if (rs_val > 0)
       target = pc + offset;
     else
       target = pc + 8;
-  } else if (!strcasecmp(op_name, "BLEZL") || !strcasecmp(op_name, "BLEZ")) {
+  } else if (op_name.equals_insensitive("BLEZL") ||
+             op_name.equals_insensitive("BLEZ")) {
     if (rs_val <= 0)
       target = pc + offset;
     else
@@ -2095,7 +2098,7 @@ bool EmulateInstructionMIPS::Emulate_BXX_2ops_C(llvm::MCInst &insn) {
   uint32_t rs;
   int32_t offset, pc, target = 0;
   int32_t rs_val;
-  const char *op_name = m_insn_info->getName(insn.getOpcode()).data();
+  llvm::StringRef op_name = m_insn_info->getName(insn.getOpcode());
   uint32_t current_inst_size = m_insn_info->get(insn.getOpcode()).getSize();
 
   rs = m_reg_info->getEncodingValue(insn.getOperand(0).getReg());
@@ -2110,32 +2113,32 @@ bool EmulateInstructionMIPS::Emulate_BXX_2ops_C(llvm::MCInst &insn) {
   if (!success)
     return false;
 
-  if (!strcasecmp(op_name, "BLTZC")) {
+  if (op_name.equals_insensitive("BLTZC")) {
     if (rs_val < 0)
       target = pc + offset;
     else
       target = pc + 4;
-  } else if (!strcasecmp(op_name, "BLEZC")) {
+  } else if (op_name.equals_insensitive("BLEZC")) {
     if (rs_val <= 0)
       target = pc + offset;
     else
       target = pc + 4;
-  } else if (!strcasecmp(op_name, "BGEZC")) {
+  } else if (op_name.equals_insensitive("BGEZC")) {
     if (rs_val >= 0)
       target = pc + offset;
     else
       target = pc + 4;
-  } else if (!strcasecmp(op_name, "BGTZC")) {
+  } else if (op_name.equals_insensitive("BGTZC")) {
     if (rs_val > 0)
       target = pc + offset;
     else
       target = pc + 4;
-  } else if (!strcasecmp(op_name, "BEQZC")) {
+  } else if (op_name.equals_insensitive("BEQZC")) {
     if (rs_val == 0)
       target = pc + offset;
     else
       target = pc + 4;
-  } else if (!strcasecmp(op_name, "BNEZC")) {
+  } else if (op_name.equals_insensitive("BNEZC")) {
     if (rs_val != 0)
       target = pc + offset;
     else
@@ -2181,7 +2184,7 @@ bool EmulateInstructionMIPS::Emulate_Branch_MM(llvm::MCInst &insn) {
   bool success = false;
   int32_t target = 0;
   uint32_t current_inst_size = m_insn_info->get(insn.getOpcode()).getSize();
-  const char *op_name = m_insn_info->getName(insn.getOpcode()).data();
+  llvm::StringRef op_name = m_insn_info->getName(insn.getOpcode());
   bool update_ra = false;
   uint32_t ra_offset = 0;
 
@@ -2215,33 +2218,33 @@ bool EmulateInstructionMIPS::Emulate_Branch_MM(llvm::MCInst &insn) {
   if (!success)
     return false;
 
-  if (!strcasecmp(op_name, "BEQZ16_MM")) {
+  if (op_name.equals_insensitive("BEQZ16_MM")) {
     if (rs_val == 0)
       target = pc + offset;
     else
       target = pc + current_inst_size +
                m_next_inst_size; // Skip delay slot instruction.
-  } else if (!strcasecmp(op_name, "BNEZ16_MM")) {
+  } else if (op_name.equals_insensitive("BNEZ16_MM")) {
     if (rs_val != 0)
       target = pc + offset;
     else
       target = pc + current_inst_size +
                m_next_inst_size; // Skip delay slot instruction.
-  } else if (!strcasecmp(op_name, "BEQZC_MM")) {
+  } else if (op_name.equals_insensitive("BEQZC_MM")) {
     if (rs_val == 0)
       target = pc + 4 + offset;
     else
       target =
           pc +
           4; // 32 bit instruction and does not have delay slot instruction.
-  } else if (!strcasecmp(op_name, "BNEZC_MM")) {
+  } else if (op_name.equals_insensitive("BNEZC_MM")) {
     if (rs_val != 0)
       target = pc + 4 + offset;
     else
       target =
           pc +
           4; // 32 bit instruction and does not have delay slot instruction.
-  } else if (!strcasecmp(op_name, "BGEZALS_MM")) {
+  } else if (op_name.equals_insensitive("BGEZALS_MM")) {
     if (rs_val >= 0)
       target = pc + offset;
     else
@@ -2249,7 +2252,7 @@ bool EmulateInstructionMIPS::Emulate_Branch_MM(llvm::MCInst &insn) {
 
     update_ra = true;
     ra_offset = 6;
-  } else if (!strcasecmp(op_name, "BLTZALS_MM")) {
+  } else if (op_name.equals_insensitive("BLTZALS_MM")) {
     if (rs_val >= 0)
       target = pc + offset;
     else
@@ -2281,7 +2284,7 @@ bool EmulateInstructionMIPS::Emulate_Branch_MM(llvm::MCInst &insn) {
 bool EmulateInstructionMIPS::Emulate_JALRx16_MM(llvm::MCInst &insn) {
   bool success = false;
   uint32_t ra_offset = 0;
-  const char *op_name = m_insn_info->getName(insn.getOpcode()).data();
+  llvm::StringRef op_name = m_insn_info->getName(insn.getOpcode());
 
   uint32_t rs = m_reg_info->getEncodingValue(insn.getOperand(0).getReg());
 
@@ -2295,9 +2298,9 @@ bool EmulateInstructionMIPS::Emulate_JALRx16_MM(llvm::MCInst &insn) {
   if (!success)
     return false;
 
-  if (!strcasecmp(op_name, "JALR16_MM"))
+  if (op_name.equals_insensitive("JALR16_MM"))
     ra_offset = 6; // 2-byte instruction with 4-byte delay slot.
-  else if (!strcasecmp(op_name, "JALRS16_MM"))
+  else if (op_name.equals_insensitive("JALRS16_MM"))
     ra_offset = 4; // 2-byte instruction with 2-byte delay slot.
 
   Context context;
@@ -2320,7 +2323,7 @@ bool EmulateInstructionMIPS::Emulate_JALRx16_MM(llvm::MCInst &insn) {
 bool EmulateInstructionMIPS::Emulate_JALx(llvm::MCInst &insn) {
   bool success = false;
   uint32_t offset = 0, target = 0, pc = 0, ra_offset = 0;
-  const char *op_name = m_insn_info->getName(insn.getOpcode()).data();
+  llvm::StringRef op_name = m_insn_info->getName(insn.getOpcode());
 
   /*
    * JALS target
@@ -2339,11 +2342,11 @@ bool EmulateInstructionMIPS::Emulate_JALx(llvm::MCInst &insn) {
     return false;
 
   // These are PC-region branches and not PC-relative.
-  if (!strcasecmp(op_name, "JALS_MM")) {
+  if (op_name.equals_insensitive("JALS_MM")) {
     // target address is in the “current” 128 MB-aligned region
     target = (pc & 0xF8000000UL) | offset;
     ra_offset = 6;
-  } else if (!strcasecmp(op_name, "JALX_MM")) {
+  } else if (op_name.equals_insensitive("JALX_MM")) {
     // target address is in the “current” 256 MB-aligned region
     target = (pc & 0xF0000000UL) | offset;
     ra_offset = 8;
@@ -2668,7 +2671,7 @@ bool EmulateInstructionMIPS::Emulate_FP_branch(llvm::MCInst &insn) {
   bool success = false;
   uint32_t cc, fcsr;
   int32_t pc, offset, target = 0;
-  const char *op_name = m_insn_info->getName(insn.getOpcode()).data();
+  llvm::StringRef op_name = m_insn_info->getName(insn.getOpcode());
 
   cc = m_reg_info->getEncodingValue(insn.getOperand(0).getReg());
   offset = insn.getOperand(1).getImm();
@@ -2684,12 +2687,14 @@ bool EmulateInstructionMIPS::Emulate_FP_branch(llvm::MCInst &insn) {
   /* fcsr[23], fcsr[25-31] are vaild condition bits */
   fcsr = ((fcsr >> 24) & 0xfe) | ((fcsr >> 23) & 0x01);
 
-  if (!strcasecmp(op_name, "BC1F") || !strcasecmp(op_name, "BC1FL")) {
+  if (op_name.equals_insensitive("BC1F") ||
+      op_name.equals_insensitive("BC1FL")) {
     if ((fcsr & (1 << cc)) == 0)
       target = pc + offset;
     else
       target = pc + 8;
-  } else if (!strcasecmp(op_name, "BC1T") || !strcasecmp(op_name, "BC1TL")) {
+  } else if (op_name.equals_insensitive("BC1T") ||
+             op_name.equals_insensitive("BC1TL")) {
     if ((fcsr & (1 << cc)) != 0)
       target = pc + offset;
     else
@@ -2784,7 +2789,7 @@ bool EmulateInstructionMIPS::Emulate_3D_branch(llvm::MCInst &insn) {
   bool success = false;
   uint32_t cc, fcsr;
   int32_t pc, offset, target = 0;
-  const char *op_name = m_insn_info->getName(insn.getOpcode()).data();
+  llvm::StringRef op_name = m_insn_info->getName(insn.getOpcode());
 
   cc = m_reg_info->getEncodingValue(insn.getOperand(0).getReg());
   offset = insn.getOperand(1).getImm();
@@ -2801,25 +2806,25 @@ bool EmulateInstructionMIPS::Emulate_3D_branch(llvm::MCInst &insn) {
   /* fcsr[23], fcsr[25-31] are vaild condition bits */
   fcsr = ((fcsr >> 24) & 0xfe) | ((fcsr >> 23) & 0x01);
 
-  if (!strcasecmp(op_name, "BC1ANY2F")) {
+  if (op_name.equals_insensitive("BC1ANY2F")) {
     /* if any one bit is 0 */
     if (((fcsr >> cc) & 3) != 3)
       target = pc + offset;
     else
       target = pc + 8;
-  } else if (!strcasecmp(op_name, "BC1ANY2T")) {
+  } else if (op_name.equals_insensitive("BC1ANY2T")) {
     /* if any one bit is 1 */
     if (((fcsr >> cc) & 3) != 0)
       target = pc + offset;
     else
       target = pc + 8;
-  } else if (!strcasecmp(op_name, "BC1ANY4F")) {
+  } else if (op_name.equals_insensitive("BC1ANY4F")) {
     /* if any one bit is 0 */
     if (((fcsr >> cc) & 0xf) != 0xf)
       target = pc + offset;
     else
       target = pc + 8;
-  } else if (!strcasecmp(op_name, "BC1ANY4T")) {
+  } else if (op_name.equals_insensitive("BC1ANY4T")) {
     /* if any one bit is 1 */
     if (((fcsr >> cc) & 0xf) != 0)
       target = pc + offset;
