@@ -11,11 +11,8 @@ from lldbsuite.test import lldbutil
 
 
 class TestBreakpointInGlobalConstructors(TestBase):
-
-    mydir = TestBase.compute_mydir(__file__)
     NO_DEBUG_INFO_TESTCASE = True
 
-    @expectedFailureNetBSD
     def test(self):
         self.build()
         self.line_foo = line_number('foo.cpp', '// !BR_foo')
