@@ -15,8 +15,6 @@ from lldbsuite.test import lldbutil
 
 class CommandLineCompletionTestCase(TestBase):
 
-    mydir = TestBase.compute_mydir(__file__)
-
     NO_DEBUG_INFO_TESTCASE = True
 
     @classmethod
@@ -43,7 +41,7 @@ class CommandLineCompletionTestCase(TestBase):
 
         (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(self,
                                           '// Break here', self.main_source_spec)
-        self.assertEquals(process.GetState(), lldb.eStateStopped)
+        self.assertState(process.GetState(), lldb.eStateStopped)
 
         # Since CommandInterpreter has been corrected to update the current execution
         # context at the beginning of HandleCompletion, we're here explicitly testing
@@ -131,7 +129,6 @@ class CommandLineCompletionTestCase(TestBase):
 
 
     @skipIfRemote
-    @skipIfReproducer
     def test_common_completion_process_pid_and_name(self):
         # The LLDB process itself and the process already attached to are both
         # ignored by the process discovery mechanism, thus we need a process known
@@ -234,7 +231,7 @@ class CommandLineCompletionTestCase(TestBase):
 
     def test_log_file(self):
         # Complete in our source directory which contains a 'main.cpp' file.
-        src_dir =  os.path.dirname(os.path.realpath(__file__)) + '/'
+        src_dir =  self.getSourceDir() + '/'
         self.complete_from_to('log enable lldb expr -f ' + src_dir,
                               ['main.cpp'])
 
@@ -294,6 +291,7 @@ class CommandLineCompletionTestCase(TestBase):
         self.complete_from_to('help watchpoint s', 'help watchpoint set ')
 
     @expectedFailureNetBSD
+    @add_test_categories(["watchpoint"])
     def test_common_complete_watchpoint_ids(self):
         subcommands = ['enable', 'disable', 'delete', 'modify', 'ignore']
 
@@ -397,7 +395,7 @@ class CommandLineCompletionTestCase(TestBase):
     def test_settings_set_target_process_dot(self):
         """Test that 'settings set target.process.t' completes to 'settings set target.process.thread.'."""
         self.complete_from_to(
-            'settings set target.process.t',
+            'settings set target.process.thr',
             'settings set target.process.thread.')
 
     def test_settings_set_target_process_thread_dot(self):
@@ -511,7 +509,7 @@ class CommandLineCompletionTestCase(TestBase):
 
     def test_command_script_delete(self):
         self.runCmd("command script add -h test_desc -f none -s current usercmd1")
-        self.check_completion_with_desc('command script delete ', [['usercmd1', 'test_desc']])
+        self.check_completion_with_desc('command script delete ', [['usercmd1', '']])
 
     def test_command_delete(self):
         self.runCmd(r"command regex test_command s/^$/finish/ 's/([0-9]+)/frame select %1/'")
@@ -678,7 +676,7 @@ class CommandLineCompletionTestCase(TestBase):
 
         self.build()
         self.dbg.CreateTarget(self.getBuildArtifact("a.out"))
-        self.runCmd('target stop-hook add test DONE')
+        self.runCmd('target stop-hook add -o test')
 
         for subcommand in subcommands:
             self.complete_from_to('target stop-hook ' + subcommand + ' ',
