@@ -9,12 +9,10 @@ from lldbsuite.test.lldbtest import TestBase
 @skipIfWindows
 class ConcurrentManyCrash(ConcurrentEventsBase):
 
-    mydir = ConcurrentEventsBase.compute_mydir(__file__)
-
     # Atomic sequences are not supported yet for MIPS in LLDB.
     @skipIf(triple='^mips')
     @skipIfOutOfTreeDebugserver
     def test(self):
         """Test 100 threads that cause a segfault."""
-        self.build(dictionary=self.getBuildFlags())
+        self.build()
         self.do_thread_actions(num_crash_threads=100)
