@@ -5,8 +5,7 @@ import lldbsuite.test.lldbutil as lldbutil
 
 class TestObjCXXBridgedPO(TestBase):
 
-    mydir = TestBase.compute_mydir(__file__)
-
+    @expectedFailureAll(macos_version=[">=", "13.0"]) # rdar://96224141
     def test_bridged_type_po(self):
         self.build()
         lldbutil.run_to_source_breakpoint(
