@@ -190,14 +190,14 @@ TEST_F(GDBRemoteCommunicationClientTest, GetModulesInfo) {
       R"("file_path":"/foo/bar.so","file_offset":0,"file_size":1234}]])");
 
   auto result = async_result.get();
-  ASSERT_TRUE(result.hasValue());
+  ASSERT_TRUE(result.has_value());
   ASSERT_EQ(1u, result->size());
-  EXPECT_EQ("/foo/bar.so", result.getValue()[0].GetFileSpec().GetPath());
-  EXPECT_EQ(triple, result.getValue()[0].GetArchitecture().GetTriple());
+  EXPECT_EQ("/foo/bar.so", result.value()[0].GetFileSpec().GetPath());
+  EXPECT_EQ(triple, result.value()[0].GetArchitecture().GetTriple());
   EXPECT_EQ(UUID::fromData("@ABCDEFGHIJKLMNO", 16),
-            result.getValue()[0].GetUUID());
-  EXPECT_EQ(0u, result.getValue()[0].GetObjectOffset());
-  EXPECT_EQ(1234u, result.getValue()[0].GetObjectSize());
+            result.value()[0].GetUUID());
+  EXPECT_EQ(0u, result.value()[0].GetObjectOffset());
+  EXPECT_EQ(1234u, result.value()[0].GetObjectSize());
 }
 
 TEST_F(GDBRemoteCommunicationClientTest, GetModulesInfo_UUID20) {
@@ -215,14 +215,14 @@ TEST_F(GDBRemoteCommunicationClientTest, GetModulesInfo_UUID20) {
       R"("file_path":"/foo/bar.so","file_offset":0,"file_size":1234}]])");
 
   auto result = async_result.get();
-  ASSERT_TRUE(result.hasValue());
+  ASSERT_TRUE(result.has_value());
   ASSERT_EQ(1u, result->size());
-  EXPECT_EQ("/foo/bar.so", result.getValue()[0].GetFileSpec().GetPath());
-  EXPECT_EQ(triple, result.getValue()[0].GetArchitecture().GetTriple());
+  EXPECT_EQ("/foo/bar.so", result.value()[0].GetFileSpec().GetPath());
+  EXPECT_EQ(triple, result.value()[0].GetArchitecture().GetTriple());
   EXPECT_EQ(UUID::fromData("@ABCDEFGHIJKLMNOPQRS", 20),
-            result.getValue()[0].GetUUID());
-  EXPECT_EQ(0u, result.getValue()[0].GetObjectOffset());
-  EXPECT_EQ(1234u, result.getValue()[0].GetObjectSize());
+            result.value()[0].GetUUID());
+  EXPECT_EQ(0u, result.value()[0].GetObjectOffset());
+  EXPECT_EQ(1234u, result.value()[0].GetObjectSize());
 }
 
 TEST_F(GDBRemoteCommunicationClientTest, GetModulesInfoInvalidResponse) {
