@@ -12,6 +12,7 @@
 //===----------------------------------------------------------------------===//
 
 #include "ReduceGlobalVars.h"
+#include "Utils.h"
 #include "llvm/IR/Constants.h"
 #include <set>
 
@@ -40,7 +41,7 @@ static void extractGVsFromModule(Oracle &O, Module &Program) {
         if (auto *Inst = dyn_cast<Instruction>(U))
           InstToRemove.push_back(Inst);
 
-      GV.replaceAllUsesWith(UndefValue::get(GV.getType()));
+      GV.replaceAllUsesWith(getDefaultValue(GV.getType()));
       ToRemove.push_back(&GV);
     }
 
@@ -49,7 +50,7 @@ static void extractGVsFromModule(Oracle &O, Module &Program) {
     if (!V)
       continue;
     auto *Inst = cast<Instruction>(V);
-    Inst->replaceAllUsesWith(UndefValue::get(Inst->getType()));
+    Inst->replaceAllUsesWith(getDefaultValue(Inst->getType()));
     Inst->eraseFromParent();
   }
 
