@@ -91,8 +91,8 @@ debug_pubtypes:
   DWARFYAML::Data Data;
   ASSERT_THAT_ERROR(parseDWARFYAML(Yaml, Data), Succeeded());
 
-  ASSERT_TRUE(Data.PubNames.hasValue());
-  DWARFYAML::PubSection PubNames = Data.PubNames.getValue();
+  ASSERT_TRUE(Data.PubNames.has_value());
+  DWARFYAML::PubSection PubNames = Data.PubNames.value();
 
   ASSERT_EQ(PubNames.Entries.size(), 2u);
   EXPECT_EQ((uint32_t)PubNames.Entries[0].DieOffset, 0x1234u);
@@ -100,8 +100,8 @@ debug_pubtypes:
   EXPECT_EQ((uint32_t)PubNames.Entries[1].DieOffset, 0x4321u);
   EXPECT_EQ(PubNames.Entries[1].Name, "def");
 
-  ASSERT_TRUE(Data.PubTypes.hasValue());
-  DWARFYAML::PubSection PubTypes = Data.PubTypes.getValue();
+  ASSERT_TRUE(Data.PubTypes.has_value());
+  DWARFYAML::PubSection PubTypes = Data.PubTypes.value();
 
   ASSERT_EQ(PubTypes.Entries.size(), 2u);
   EXPECT_EQ((uint32_t)PubTypes.Entries[0].DieOffset, 0x1234u);
@@ -157,8 +157,8 @@ debug_gnu_pubtypes:
   DWARFYAML::Data Data;
   ASSERT_THAT_ERROR(parseDWARFYAML(Yaml, Data), Succeeded());
 
-  ASSERT_TRUE(Data.GNUPubNames.hasValue());
-  DWARFYAML::PubSection GNUPubNames = Data.GNUPubNames.getValue();
+  ASSERT_TRUE(Data.GNUPubNames.has_value());
+  DWARFYAML::PubSection GNUPubNames = Data.GNUPubNames.value();
 
   ASSERT_EQ(GNUPubNames.Entries.size(), 2u);
   EXPECT_EQ((uint32_t)GNUPubNames.Entries[0].DieOffset, 0x1234u);
@@ -168,8 +168,8 @@ debug_gnu_pubtypes:
   EXPECT_EQ((uint8_t)GNUPubNames.Entries[1].Descriptor, 0x34);
   EXPECT_EQ(GNUPubNames.Entries[1].Name, "def");
 
-  ASSERT_TRUE(Data.GNUPubTypes.hasValue());
-  DWARFYAML::PubSection GNUPubTypes = Data.GNUPubTypes.getValue();
+  ASSERT_TRUE(Data.GNUPubTypes.has_value());
+  DWARFYAML::PubSection GNUPubTypes = Data.GNUPubTypes.value();
 
   ASSERT_EQ(GNUPubTypes.Entries.size(), 2u);
   EXPECT_EQ((uint32_t)GNUPubTypes.Entries[0].DieOffset, 0x1234u);
