@@ -94,7 +94,7 @@ protected:
   std::unique_ptr<Record> Rec;
 };
 
-TYPED_TEST_CASE_P(RoundTripTest);
+TYPED_TEST_SUITE_P(RoundTripTest);
 
 template <class T> class RoundTripTestV5 : public ::testing::Test {
 public:
@@ -117,7 +117,7 @@ protected:
   std::unique_ptr<Record> Rec;
 };
 
-TYPED_TEST_CASE_P(RoundTripTestV5);
+TYPED_TEST_SUITE_P(RoundTripTestV5);
 
 // This test ensures that the writing and reading implementations are in sync --
 // that given write(read(write(R))) == R.
@@ -160,7 +160,7 @@ TYPED_TEST_P(RoundTripTest, RoundTripsSingleValue) {
   EXPECT_THAT(Records[1]->getRecordType(), Eq(R->getRecordType()));
 }
 
-REGISTER_TYPED_TEST_CASE_P(RoundTripTest, RoundTripsSingleValue);
+REGISTER_TYPED_TEST_SUITE_P(RoundTripTest, RoundTripsSingleValue);
 
 // We duplicate the above case for the V5 version using different types and
 // encodings.
@@ -201,21 +201,21 @@ TYPED_TEST_P(RoundTripTestV5, RoundTripsSingleValue) {
   EXPECT_THAT(Records[1]->getRecordType(), Eq(R->getRecordType()));
 }
 
-REGISTER_TYPED_TEST_CASE_P(RoundTripTestV5, RoundTripsSingleValue);
+REGISTER_TYPED_TEST_SUITE_P(RoundTripTestV5, RoundTripsSingleValue);
 
 // These are the record types we support for v4 and below.
 using RecordTypes =
     ::testing::Types<NewBufferRecord, NewCPUIDRecord, TSCWrapRecord,
                      WallclockRecord, CustomEventRecord, CallArgRecord,
                      PIDRecord, FunctionRecord>;
-INSTANTIATE_TYPED_TEST_CASE_P(Records, RoundTripTest, RecordTypes);
+INSTANTIATE_TYPED_TEST_SUITE_P(Records, RoundTripTest, RecordTypes, );
 
 // For V5, we have two new types we're supporting.
 using RecordTypesV5 =
     ::testing::Types<NewBufferRecord, NewCPUIDRecord, TSCWrapRecord,
                      WallclockRecord, CustomEventRecordV5, TypedEventRecord,
                      CallArgRecord, PIDRecord, FunctionRecord>;
-INSTANTIATE_TYPED_TEST_CASE_P(Records, RoundTripTestV5, RecordTypesV5);
+INSTANTIATE_TYPED_TEST_SUITE_P(Records, RoundTripTestV5, RecordTypesV5, );
 
 } // namespace
 } // namespace xray
