@@ -12,7 +12,7 @@
 #define LLVM_UNITTESTS_TOOLS_LLVMEXEGESIS_X86_TESTBASE_H
 
 #include "LlvmState.h"
-#include "llvm/Support/TargetRegistry.h"
+#include "llvm/MC/TargetRegistry.h"
 #include "llvm/Support/TargetSelect.h"
 #include "gmock/gmock.h"
 #include "gtest/gtest.h"
