@@ -9,7 +9,6 @@
 #ifndef MLIR_DIALECT_MEMREF_IR_MEMREF_H_
 #define MLIR_DIALECT_MEMREF_IR_MEMREF_H_
 
-#include "mlir/Dialect/Arithmetic/IR/Arithmetic.h"
 #include "mlir/Dialect/Utils/ReshapeOpsUtils.h"
 #include "mlir/IR/Dialect.h"
 #include "mlir/Interfaces/CallInterfaces.h"
@@ -21,6 +20,11 @@
 
 namespace mlir {
 
+namespace arith {
+enum class AtomicRMWKind : uint64_t;
+class AtomicRMWKindAttr;
+} // namespace arith
+
 class Location;
 class OpBuilder;
 
@@ -43,6 +47,11 @@ LogicalResult foldMemRefCast(Operation *op, Value inner = nullptr);
 /// type.
 Type getTensorTypeFromMemRefType(Type type);
 
+/// Finds a single dealloc operation for the given allocated value. If there
+/// are > 1 deallocates for `allocValue`, returns None, else returns the single
+/// deallocate if it exists or nullptr.
+Optional<Operation *> findDealloc(Value allocValue);
+
 } // namespace memref
 } // namespace mlir
 
